"""
w2lapp.add.py: add an entry

web2ldap - a web-based LDAP Client,
see http://www.web2ldap.de for details

(c) by Michael Stroeder <michael@stroeder.com>

This module is distributed under the terms of the
GPL (GNU GENERAL PUBLIC LICENSE) Version 2
(see http://www.gnu.org/copyleft/gpl.html)

$Id: add.py,v 1.63 2006/03/13 19:15:46 michael Exp $
"""

import ldap, ldap.modlist,pyweblib.forms, \
       w2lapp.cnf,w2lapp.core,w2lapp.gui,w2lapp.addmodifyform,w2lapp.modify

from ldaputil.base import escape_dn_chars,is_dn,ParentDN,SplitRDN

def ModlistTable(schema,modlist):
  """
  Return a string containing a HTML table showing attr type/value pairs
  """
  s = []
  s.append('<table summary="Modify list">')
  for attr_type,attr_value in modlist:
    if w2lapp.schema.no_humanreadable_attr(schema,attr_type):
      tablestr = '%s bytes of binary data' % (
        ' + '.join(
          [ str(len(x)) for x in attr_value ]
        )
      )
    else:
      tablestr = '<br>'.join([
        pyweblib.forms.escapeHTML(repr(v))
        for v in attr_value
      ])
    s.append('<tr><td>%s</td><td>%s</td></tr>' % (
      pyweblib.forms.escapeHTML(attr_type),tablestr
    )
  )
  s.append('</table>')
  return '\n'.join(s) # ModlistTable()


def SearchMissingParentEntries(ls,new_dn):
  parent_dn = ParentDN(new_dn)
  missing_parentdns = []
  while parent_dn.lower():
    try:
      ls.l.search_s(
        parent_dn.encode('utf-8'),ldap.SCOPE_BASE,
        '(objectClass=*)',['objectClass']
      )
    except ldap.NO_SUCH_OBJECT:
      missing_parentdns.append(parent_dn)
    parent_dn = ParentDN(parent_dn)
  return missing_parentdns


def DisplayMissingParentEntries(sid,outf,form,ls,dn,new_dn,missing_parentdns):
  w2lapp.gui.TopSection(
    sid,outf,form,ls,dn,
    'Error adding new entry',
    w2lapp.gui.MainMenu(sid,form,ls,dn),
    context_menu_list=[]
  )
  outf.write("""<div id="ErrorMessage" class="Main">
    <h1>Error</h1>
    <p>No such object: The entry %s was not correctly added.</p>
    Missing parent entries:
    <table>""" % (
      w2lapp.gui.DisplayDN(sid,form,ls,new_dn)
    )
  )
  for parent_dn in missing_parentdns:
    rdn,dn = SplitRDN(parent_dn)
    outf.write('<tr><td>%s</td><td>%s</td></tr>' % (
	parent_dn,
	form.applAnchor(
	  'addform','Create entry',sid,
          [('dn',dn),('add_rdn',rdn)],
          target=None
        )
      )
    )
  outf.write('</table></div>')
  w2lapp.gui.PrintFooter(outf,form)


########################################################################
# Add new entry
########################################################################

def w2l_Add(sid,outf,command,form,ls,dn):

  sub_schema = ls.retrieveSubSchema(dn,w2lapp.cnf.GetParam(ls,'_schema',None))

  entry,invalid_attrs = w2lapp.modify.GetEntryfromInputForm(form,ls,dn,sub_schema)

  if 'input_formtype' in form.inputFieldNames:
    w2lapp.addmodifyform.w2l_AddForm(
      sid,outf,'addform',form,ls,dn,
      Msg='',
      entry=entry,
      skip_oc_input=1
    )
    return

  if invalid_attrs:
    w2lapp.addmodifyform.w2l_AddForm(
      sid,outf,'addform',form,ls,dn,
      Msg='Syntax check failed for the following attributes: %s' % (
        ', '.join(invalid_attrs)
      ),
      entry=entry,
      skip_oc_input=1
    )
    return

  # Final check if object classes were defined in entry data
  if not entry.has_key('objectClass'):
    raise w2lapp.core.ErrorExit('No object class(es) defined.')

  # Try to get the RDN from the user's input.
  rdn = form.getInputValue('add_rdn',[''])[0]

  # If add_rdn does not contain a complete RDN try to determine
  # the attribute type for forming the RDN.
  if is_dn(rdn):
    new_rdn_attrtype = None
  else:
    # Check for attrtype= format
    try:
      new_rdn_attrtype,value = rdn.split('=',1)
    except ValueError:
      new_rdn_attrtype = None
    else:
      if value:
        new_rdn_attrtype = None
    if new_rdn_attrtype is None:
      w2lapp.addmodifyform.w2l_AddForm(
        sid,outf,'addform',form,ls,dn,
        Msg='<p>Wrong format of RDN string.</p>',
        rdn_default=rdn,
        entry=entry,
        skip_oc_input=1
      )
      return

  # Try to define complete RDN if 
  if new_rdn_attrtype:
    new_rdn_attrtype = new_rdn_attrtype.strip()
    # Normalize old LDAPv2 RDN form
    if new_rdn_attrtype.lower().startswith('oid.'):
      new_rdn_attrtype = new_rdn_attrtype[4:]
    if entry.has_key(new_rdn_attrtype) and entry[new_rdn_attrtype]:
      new_rdn_attrvalue = entry[new_rdn_attrtype][0]
      rdn = '='.join((
        new_rdn_attrtype,
        escape_dn_chars(unicode(new_rdn_attrvalue,ls.charset))
      ))
    else:
      w2lapp.addmodifyform.w2l_AddForm(
        sid,outf,'addform',form,ls,dn,
        Msg='<p>Attribute value for <strong>%s</strong> required for default RDN not in entry data.</p>' % (new_rdn_attrtype),
        rdn_default=new_rdn_attrtype+'=',
        entry=entry,
        skip_oc_input=1
      )
      return

  if not rdn:
    w2lapp.addmodifyform.w2l_AddForm(
      sid,outf,'addform',form,ls,dn,
      Msg='<p>Unable to determine new RDN.</p>',
      rdn_default='',
      entry=entry,
      skip_oc_input=1
    )
    return

  # Modify-Liste zumsammenbauen
  modlist = ldap.modlist.addModlist(entry)

  if not modlist:
    raise w2lapp.core.ErrorExit('Cannot add entry without attribute values.')

  if dn:
    new_dn = ','.join([rdn,dn])
  else:
    # Makes it possible to add entries for a namingContext
    new_dn = rdn

  # Try to add the new entry
  try:
    ls.addEntry(new_dn,modlist)
  except ldap.NO_SUCH_OBJECT,e:
    # Probably one or more parent entries do not exist
    missing_parentdns = SearchMissingParentEntries(ls,new_dn)
    if missing_parentdns:
      DisplayMissingParentEntries(sid,outf,form,ls,dn,new_dn,missing_parentdns)
    else:
      raise e
  except (
    ldap.ALREADY_EXISTS,
    ldap.CONSTRAINT_VIOLATION,
    ldap.INVALID_DN_SYNTAX,
    ldap.INVALID_SYNTAX,
    ldap.NAMING_VIOLATION,
    ldap.OBJECT_CLASS_VIOLATION,
    ldap.OTHER,
    ldap.TYPE_OR_VALUE_EXISTS,
    ldap.UNDEFINED_TYPE,
  ),e:
    # Some error in user's input => present input form to edit input values
    w2lapp.addmodifyform.w2l_AddForm(
      sid,outf,'addform',form,ls,dn,
      Msg=w2lapp.gui.LDAPError2ErrMsg(e),
      rdn_default=rdn,
      entry=entry,
      skip_oc_input=1
    )
  else:
    # Everything seems to went fine
    w2lapp.gui.SimpleMessage(
      sid,outf,form,ls,dn,
      'Added Entry',
      'Added entry %s with your input data:\n%s' % (
        w2lapp.gui.DisplayDN(sid,form,ls,new_dn,commandbutton=1),
        ModlistTable(sub_schema,modlist)
      ),
      main_menu_list=w2lapp.gui.MainMenu(sid,form,ls,dn),
      context_menu_list=[]
    )
